

function [CI, CSI] = centroidIndex(c1, c2, labels1, labels2)

%% check input data
if ( nargin ~=4 )
    error('Two sets of centroids and two sets of labels are needed as input!\n');
end

[m1 n1] = size(labels1);
[m2 n2] = size(labels2);

if ( (m1 ~= 1) || (m2 ~= 1) || (n1 ~= n2) )
    error('Invalid input partitions!\n Two partitions should have the same length and with the size of [1 N]. \n');
end
N = n1; % number of data objects

min1 = min(labels1);
min2 = min(labels2);

% make labels start from 1 if they are started from 0
if ( min1 == 0 )
    labels1 = labels1 + 1;
end

if ( min2 == 0 )
    labels2 = labels2 + 1;
end

K1 = max(labels1); 
K2 = max(labels2); 
m1 = unique(labels1); m2 = unique(labels2);

% if empty clusters, just fix it
if ( length(m1) ~= K1 )
    for i=1:length(m1)
        labels1(labels1==m1(i)) = i;
    end
end

% if empty clusters, just fix it
if ( length(m2) ~= K2 )
    for i=1:length(m2)
        labels2(labels2==m2(i)) = i;
    end
end

K1 = length(m1);
K2 = length(m2);

% check centroids

d1 = size(c1,2);
d2 = size(c2,2);

if d1 ~= d2
    error('Invalid input centroids!\nCentroids of two clusterings are of different dimensions');
end

% remove centroids with NaN
[row col] = find(isnan(c1)); row=unique(row);
c1(row,:) = [];
[row col] = find(isnan(c2)); row=unique(row);
c2(row,:) = [];

t1 = size(c1,1);
t2 = size(c2,1);

if (t1~=K1 || t2~=K2)
    error('Mismatch between centroids and labels!\n');
end

%% calculating centroid index and centroid similarity index
map12 = mapCentroids(c1,c2); % mapping c1 centroids to c2
map21 = mapCentroids(c2,c1); % mapping c2 centroids to c1
orphan2 = K2 - length(unique(map12));
orphan1 = K1 - length(unique(map21));

CI = max(orphan1, orphan2);

% Contingency matrix
cntg_mat = zeros(K2,K1);
for i = 1:N
   cntg_mat(labels2(i),labels1(i)) = cntg_mat(labels2(i),labels1(i)) + 1;
end

s12 = 0;
for i = 1:K1
    j = map12(i);
    s12 = s12 + cntg_mat(j,i);
end
s12 = s12 / N;

s21 = 0;
for j = 1:K2
    i = map21(j);
    s21 = s21 + cntg_mat(j,i);
end
s21 = s21 / N;

CSI = (s12+s21)/2;




