%% Mohammad Rezaei
% Date: 29.8.2018
% external validity types: 'RI' 'ARI' 'MI' 'NMI' 'NVD' 'PSI' 'CI' 'CSI'
% Updated: 9.10.2018 

function [RI, ARI, NMI, NVD, PSI] = getExternalIndices(labels1, labels2)


%% check input data
if ( nargin ~=2 )
    error('Two partitions are needed as input!\n');
end

[m1 n1] = size(labels1);
[m2 n2] = size(labels2);

if ( (m1 ~= 1) || (m2 ~= 1) || (n1 ~= n2) )
    error('Invalid input partitions!\n Two partitions should have the same length and with the size of [1 N]. \n');
end
N = n1; % number of data objects

min1 = min(labels1);
min2 = min(labels2);

% make labels start from 1 if they are started from 0
if ( min1 == 0 )
    labels1 = labels1 + 1;
end

if ( min2 == 0 )
    labels2 = labels2 + 1;
end

K1 = max(labels1); 
K2 = max(labels2); 
m1 = unique(labels1); m2 = unique(labels2);

% if empty clusters, just fix it
if ( length(m1) ~= K1 )
    for i=1:length(m1)
        labels1(labels1==m1(i)) = i;
    end
end

% if empty clusters, just fix it
if ( length(m2) ~= K2 )
    for i=1:length(m2)
        labels2(labels2==m2(i)) = i;
    end
end

K1 = length(m1);
K2 = length(m2);

%% Contingency matrix
cntg_mat = zeros(K2,K1);
for i = 1:N
   cntg_mat(labels2(i),labels1(i)) = cntg_mat(labels2(i),labels1(i)) + 1;
end

%% RI, Mirkin, HI, DRI, ARI, DARI
ni = sum(sum(cntg_mat,2).^2);
nj = sum(sum(cntg_mat,1).^2);

type1 = 0;
for i = 1:K2
    for j = 1 : K1 
        type1 = type1 + cntg_mat(i,j)*(cntg_mat(i,j)-1);
    end
end
type1 = type1 / 2;

t1 = N*(N-1)/2;
t2 = sum(sum(cntg_mat.^2));      % sum(nij^2)
t3 = .5*(ni+nj);

nc = (N*(N^2+1)-(N+1)*ni-(N+1)*nj+2*(ni*nj)/N)/(2*(N-1)); % Expected value of RI

A = t1+t2-t3;  % agreements
RI = A/t1;

if t1==nc
   ARI=0;			
else
   ARI=(A-nc)/(t1-nc);
end

%% MI, NMI1, NMI2, VI, NVI
if K1>1 
    a = sum(cntg_mat');
else
    a=cntg_mat';
end;
if K2>1 
    b = sum(cntg_mat);
else
    b=cntg_mat;
end;

H1 = -(a/N)*log2(a/N)';
H2 = -(b/N)*log2(b/N)';

MI=0;
for i=1:K2
    for j=1:K1
        if cntg_mat(i,j)>0 
            MI=MI+cntg_mat(i,j)*log2(cntg_mat(i,j)*N/(a(i)*b(j)));
        end;
    end
end
MI = MI/N;

NMI1 = 2*MI/(H1+H2);  
NMI2 = MI/sqrt(H1*H2);

if isnan(NMI2)
    NMI2 = 0;
end

if isnan(NMI1)
    NMI1 = 0;
end

VI = H1+H2-2*MI;
NVI = VI/(H1+H2);

NMI = NMI1;

%% NVD Normalized Van Dongen and purity
nij12 = 0;
for i=1:K1
   nij12 = nij12 + max(cntg_mat(:,i));
end
n1dot = sum(cntg_mat,1); n1max = max(n1dot);

nij21 = 0;
for i=1:K2
   nij21 = nij21 + max(cntg_mat(i,:));
end
n2dot = sum(cntg_mat,2); n2max = max(n2dot);

VD = (2*N-nij12-nij21);
NVD = VD / (2*N);
NVD2 = VD / (2*N-n1max-n2max); % I think it is like adjusted for chance

%% Criterion H
nij = 0;
cntg_mat_t = cntg_mat;
for i=1:K1
   [max_v II] = max(cntg_mat_t(:));
   if ( max_v == 0 )
     break;
   else
     nij = nij + max_v;
     [IR, IC]=ind2sub(size(cntg_mat_t),II);
     % to exclude the paired clusters for next selection of pairs
     cntg_mat_t(IR,:) = 0; 
     cntg_mat_t(:,IC) = 0;
   end
end
CH = 1-nij/N;

%% PSI 
PSI = 0;
sx = pairPartitionsSimilarity(labels1, labels2); % similarity of two clusters
ex = pairPartitionsExpected(labels1, labels2); % expected value of the index
if ( sx > ex )
    PSI = (sx - ex)/(double(max(K1, K2)) - ex); % adjustment
end

% simplified version
% if ( sx > 1 )
%     PSI = (sx - 1)/(K - 1); % adjustment
% end