%% Mohammad Rezaei
% Date: 29.8.2018
% external validity types: 'RI' 'ARI' 'MI' 'NMI' 'NVD' 'PSI' 'CI' 'CSI'
% Updated: 9.10.2018 

function PSI = getPSI(labels1, labels2)


%% check input data
if ( nargin ~=2 )
    error('Two partitions are needed as input!\n');
end

[m1 n1] = size(labels1);
[m2 n2] = size(labels2);

if ( (m1 ~= 1) || (m2 ~= 1) || (n1 ~= n2) )
    error('Invalid input partitions!\n Two partitions should have the same length and with the size of [1 N]. \n');
end
N = n1; % number of data objects

min1 = min(labels1);
min2 = min(labels2);

% make labels start from 1 if they are started from 0
if ( min1 == 0 )
    labels1 = labels1 + 1;
end

if ( min2 == 0 )
    labels2 = labels2 + 1;
end

K1 = max(labels1);
K2 = max(labels2);
m1 = unique(labels1); m2 = unique(labels2);

% if empty clusters, just fix it
if ( length(m1) ~= K1 )
    for i=1:length(m1)
        labels1(labels1==m1(i)) = i;
    end
end

% if empty clusters, just fix it
if ( length(m2) ~= K2 )
    for i=1:length(m2)
        labels2(labels2==m2(i)) = i;
    end
end

K1 = length(m1);
K2 = length(m2);

%% PSI 
PSI = 0;
sx = pairPartitionsSimilarity(labels1, labels2); % similarity of two clusters
%ex = pairPartitionsExpected(labels1, labels2); % expected value of the index
ex = 1;
if ( sx > ex )
    PSI = (sx - ex)/(double(max(K1, K2)) - ex); % adjustment
end
