% classification-based strategy


%% Mohammad Rezaei
% Date: 8.10.2018

clc;
close all;
clearvars;

dataset = 'dim032'; % it is also the fodler that s1 dataset is in
folder_test = '..\experiments_results\dim032\dim032_rs\';
folder_train = '..\experiments_results\dim032\dim032_train_rs\';
clus_method = 'rs';
n1 = 2; % the range for the number of clusters
n2 = 25;
nSubsets = 100;
th_lastLocalMax = 0.9;

%% read full dataset
% fp = fopen(strcat('dataset\',dataset, '\', dataset, '.txt'), 'rb');
% data = textscan(fp, '%d %d');
% data = cell2mat(data);
% data = double(data);
% fclose(fp);

data = dlmread(strcat('dataset\',dataset, '\', dataset, '.txt'));

%% average similarity for each number of clusters
[RI_avg, ARI_avg, NMI_avg, NVD_avg, PSI_avg, CI_avg, CSI_avg] = deal(zeros(1,n2-n1+1));
ii = 1;
for i = n1:n2
    disp(i);
    
    for j = 1:nSubsets
        
        if ( i==17 )
            disp(j);
        end
        % read partition of the subset j
        if j < 10 
            t = strcat('0', num2str(j));
        else
            t = num2str(j);
        end
        
        % read partition of train subset
        pSubset_train = readPartition(strcat(folder_train, dataset, '_train', '-', t, '-', clus_method, '-', num2str(i), '.pa'));
        % read partition of test subset
        pSubset_test = readPartition(strcat(folder_test, dataset, '-', t, '-', clus_method, '-', num2str(i), '.pa'));
        
        %--- nearest centroid classifier ---%
        idx = readIdxFile(strcat('dataset\', dataset, '_train', '\', 'idx', '-', t, '.txt'));
        dataTrain = data(idx, :);
        % get corresponding data for the subset and calculate centroids
        idx = readIdxFile(strcat('dataset\', dataset, '\', 'idx', '-', t, '.txt'));
        dataTest = data(idx, :);
        %calculate centroids from train data
        cTrain = calcCentroidsFromIDX(dataTrain, pSubset_train);
        % find partition for test data according to the centroids of train
        % data
        D_all = EuclideanDist(cTrain', dataTest');
        [D, pSubset_test_from_train] = min(D_all, [], 1);
        %-----------------------------------%
        
        % calculate partition of fullset from each resulting partition for
        % test data; based on nearest centroid mapping
        %---- fix if there is empty partition, it makes problem later
        K1 = max(pSubset_test); m1 = unique(pSubset_test);
        % if empty clusters, just fix it
        if ( length(m1) ~= K1 )
            for i=1:length(m1)
                pSubset_test(pSubset_test==m1(i)) = i;
            end
        end
        %------------------------------------------------------------
        cTest1 = calcCentroidsFromIDX(dataTest, pSubset_test);
        %---- fix if there is empty partition, it makes problem later
        K1 = max(pSubset_test_from_train); m1 = unique(pSubset_test_from_train);
        % if empty clusters, just fix it
        if ( length(m1) ~= K1 )
            for i=1:length(m1)
                pSubset_test_from_train(pSubset_test_from_train==m1(i)) = i;
            end
        end
        %------------------------------------------------------------
        cTest2 = calcCentroidsFromIDX(dataTest, pSubset_test_from_train);
        
        D_all = EuclideanDist(cTest1', data');
        [D, pFullset1] = min(D_all, [], 1);
        D_all = EuclideanDist(cTest2', data');
        [D, pFullset2] = min(D_all, [], 1);
        
        % calculate similarity
        [RI, ARI, NMI, NVD, PSI] = getExternalIndices(pFullset1, pFullset2); NVD = 1-NVD;
        [CI, CSI] = getCI_CSI(cTest1, cTest2, pFullset1, pFullset2); CI = 1-double(CI)/i;
        RI_avg(ii) = RI_avg(ii) + RI;
        ARI_avg(ii) = ARI_avg(ii) + ARI;
        NMI_avg(ii) = NMI_avg(ii) + NMI;
        NVD_avg(ii) = NVD_avg(ii) + NVD;
        PSI_avg(ii) = PSI_avg(ii) + PSI;
        CI_avg(ii) = CI_avg(ii) + CI;
        CSI_avg(ii) = CSI_avg(ii) + CSI;
    end
    
    RI_avg(ii) = RI_avg(ii) / nSubsets;
    ARI_avg(ii) = ARI_avg(ii) / nSubsets;
    NMI_avg(ii) = NMI_avg(ii) / nSubsets;
    NVD_avg(ii) = NVD_avg(ii) / nSubsets;
    PSI_avg(ii) = PSI_avg(ii) / nSubsets;
    CI_avg(ii) = CI_avg(ii) / nSubsets;
    CSI_avg(ii) = CSI_avg(ii) / nSubsets;
    
    ii = ii + 1;
end

% plot index values when number of clusters varies
plot([n1:n2], PSI_avg);
axis([n1 n2 0.6 1]);

% last local maximum
k_RI_llm = lastLocalMax(RI_avg, th_lastLocalMax, n1);
k_ARI_llm = lastLocalMax(ARI_avg, th_lastLocalMax, n1);
k_NMI_llm = lastLocalMax(NMI_avg, th_lastLocalMax, n1);
k_PSI_llm = lastLocalMax(PSI_avg, th_lastLocalMax, n1);
k_NVD_llm = lastLocalMax(NVD_avg, th_lastLocalMax, n1);
k_CSI_llm = lastLocalMax(CSI_avg, th_lastLocalMax, n1);
k_CI_llm = lastLocalMax(CI_avg, th_lastLocalMax, n1);

[k_RI_llm k_ARI_llm k_NMI_llm k_PSI_llm k_NVD_llm k_CSI_llm k_CI_llm]
[k_RI_llm k_ARI_llm k_NMI_llm k_PSI_llm k_NVD_llm k_CSI_llm k_CI_llm]