% Cross-validation strategy: subset-fullset 

%% Mohammad Rezaei
% Date: 29.8.2018

clc;
close all;
clearvars;

dataset_orig = 'a3';
dataset = 'a3_train'; % it is also the fodler that s1 dataset is in
folder_orig_data = '..\experiments_results\a3\a3_rs\';
folder = '..\experiments_results\a3\a3_train_rs\';
clus_method = 'rs';
n1 = 40; % the range for the number of clusters
n2 = 60;
nSubsets = 100;
th_lastLocalMax = 0.9;

%% read full dataset
% fp = fopen(strcat('dataset\',dataset, '\', dataset, '.txt'), 'rb');
% data = textscan(fp, '%d %d');
% data = cell2mat(data);
% fclose(fp);

data = dlmread(strcat('dataset\',dataset, '\', dataset, '.txt'));

%% average similarity for each number of clusters
[RI_avg, ARI_avg, NMI_avg, NVD_avg, PSI_avg, CI_avg, CSI_avg] = deal(zeros(1,n2-n1+1));
ii = 1;
for i = n1:n2
    disp(i);
    
    % read partition of full dataset
    %pFullset = readPartition(strcat(folder, dataset, '-', clus_method, '-', num2str(i), '.pa'));
    pFullset = readPartition(strcat(folder_orig_data, dataset_orig, '-', clus_method, '-', num2str(i), '.pa'));
    
    for j = 1:nSubsets
        
        % read partition of the subset j
        if j < 10 
            t = strcat('0', num2str(j));
        else
            t = num2str(j);
        end
        pSubset = readPartition(strcat(folder, dataset, '-', t, '-', clus_method, '-', num2str(i), '.pa'));
        
        % get corresponding data for the subset and calculate centroids
        idx = readIdxFile(strcat('dataset\', dataset, '\', 'idx', '-', t, '.txt'));
        dataSubset = data(idx, :);
        cSubset = calcCentroidsFromIDX(dataSubset, pSubset);
        
        % calculate similarity
        pFullset_shared = pFullset(idx); % the similarity between two clusterings is calcuylated based on the shared objects between fullset and subset 
        % calculate centroids
        cFullset = calcCentroidsFromIDX(dataSubset, pFullset_shared); % we can't calculate cFullset from full dataset because in the shared data, we might miss some clusters. If we use full data to calculate the centroids, then the number of centroids might mismatch with the number of clusters
    
        [RI, ARI, NMI, NVD, PSI] = getExternalIndices(pFullset_shared', pSubset'); NVD = 1-NVD;
        [CI, CSI] = getCI_CSI(cFullset, cSubset, pFullset_shared', pSubset'); CI = 1-double(CI)/i;
        RI_avg(ii) = RI_avg(ii) + RI;
        ARI_avg(ii) = ARI_avg(ii) + ARI;
        NMI_avg(ii) = NMI_avg(ii) + NMI;
        NVD_avg(ii) = NVD_avg(ii) + NVD;
        PSI_avg(ii) = PSI_avg(ii) + PSI;
        CI_avg(ii) = CI_avg(ii) + CI;
        CSI_avg(ii) = CSI_avg(ii) + CSI;
    end
    
    RI_avg(ii) = RI_avg(ii) / nSubsets;
    ARI_avg(ii) = ARI_avg(ii) / nSubsets;
    NMI_avg(ii) = NMI_avg(ii) / nSubsets;
    NVD_avg(ii) = NVD_avg(ii) / nSubsets;
    PSI_avg(ii) = PSI_avg(ii) / nSubsets;
    CI_avg(ii) = CI_avg(ii) / nSubsets;
    CSI_avg(ii) = CSI_avg(ii) / nSubsets;
    
    ii = ii + 1;
end

% plot index values when number of clusters varies
plot([n1:n2], PSI_avg);
axis([n1 n2 0 1]);

% global maximum
[v k_RI] = max(RI_avg);k_RI=k_RI+n1-1;
[v k_ARI] = max(ARI_avg);k_ARI=k_ARI+n1-1;
[v k_NMI] = max(NMI_avg);k_NMI=k_NMI+n1-1;
[v k_PSI] = max(PSI_avg);k_PSI=k_PSI+n1-1;
[v k_NVD] = max(NVD_avg);k_NVD=k_NVD+n1-1;
[v k_CSI] = max(CSI_avg);k_CSI=k_CSI+n1-1;
[v k_CI] = max(CI_avg);k_CI=k_CI+n1-1;

[k_RI k_ARI k_NMI k_PSI k_NVD k_CSI k_CI]

% last local maximum
k_RI_llm = lastLocalMax(RI_avg, th_lastLocalMax, n1);
k_ARI_llm = lastLocalMax(ARI_avg, th_lastLocalMax, n1);
k_NMI_llm = lastLocalMax(NMI_avg, th_lastLocalMax, n1);
k_PSI_llm = lastLocalMax(PSI_avg, th_lastLocalMax, n1);
k_NVD_llm = lastLocalMax(NVD_avg, th_lastLocalMax, n1);
k_CSI_llm = lastLocalMax(CSI_avg, th_lastLocalMax, n1);
k_CI_llm = lastLocalMax(CI_avg, th_lastLocalMax, n1);

[k_RI_llm k_ARI_llm k_NMI_llm k_PSI_llm k_NVD_llm k_CSI_llm k_CI_llm]
[k_RI_llm k_ARI_llm k_NMI_llm k_PSI_llm k_NVD_llm k_CSI_llm k_CI_llm]